OBJ
  serial : "simple_serial"

PUB init(LCD_PIN, LCD_BAUD) : ok  {{ initialize the LCD driver (configured for 4 lines x 20 columns) }}
  '' turns on backlight to 1/4 power, clears lcd
  ok := serial.init(-1, LCD_PIN, LCD_BAUD)
  if not ok
    return
  serial.str(string("?G420"))
  wait100ms
  backlight(64)
  clear
  return

PUB home {{ cursor to position 0,0 }}
  serial.str(string("?a"))

PUB backspace {{ destructive backspace }}
  serial.str(string("?b"))

PUB cursor(type)
  '' Selects cursor type
  ''   0 : cursor off, blink off
  ''   1 : cursor off, blink on
  ''   2 : cursor on, blink off
  ''   3 : cursor on, blink on
  serial.str(string("?c"))
  case type
    0,1,2,3: serial.tx("0"+type)
    other: serial.tx("0")
  wait100ms

PUB clear {{ clear screen }}
  serial.str(string("?f"))

PUB backup {{ non-destructive backspace }}
  serial.str(string("?h"))

PUB forward {{ forward cursor }}
  serial.str(string("?i"))

PUB up {{ up cursor }}
  serial.str(string("?j"))

PUB down {{ down cursor }}
  serial.str(string("?k"))

PUB clearline {{ clear cursor line }}
  serial.str(string("?l"))
  wait100ms

PUB cr {{ carriage return: cursor to next line }}
  serial.str(string("?m"))

PUB nl {{ newline: CRLF, cursor to start of next line, line cleared }}
  serial.str(string("?n"))
  wait100ms

PUB set_tabs(spaces) {{ sets tab stops at every 'spaces' (must be between 1 and 9 inclusive) }}
  if lookdown(spaces : 1..9)
     serial.str(string("?t"))
     serial.tx("0"+spaces)
     wait100ms

PUB tab {{ advances cursor to next tab position }}
  serial.str(string("?t"))

PUB to_x(col) {{ puts cursor at the given column (0.19) }}
  if lookdown(col : 0..19)
     serial.str(string("?x"))
     serial.tx(col/10+48)  ' tens
     serial.tx(col//10+48) ' units

PUB to_y(line) {{ puts cursor on the given line (0..3) }}
  if lookdown(line : 0..3)
     serial.str(string("?y"))
     serial.tx(line+48)

PUB to_xy(col, line) {{ move cursor to given x,y coords }}
  if lookdown(line : 0..3)
     if lookdown(col : 0..19)
        to_x(col)
        to_y(line)

PUB backlight(intensity) {{ sets backlight intensity to the given value (0..255) }}
  intensity <#= 255
  intensity #>= 0
  serial.str(string("?B"))
  hex(intensity,2)
  wait100ms

PUB custom(char, chrDataAddr) {{ Installs custom character map }}
  '' -- chrDataAddr is address of 16-byte character definition array
  if lookdown(char : 0..7)                            ' make sure char in range
     serial.str(string("?D"))                          ' enter custom char def mode
     serial.tx("0"+char)                                ' defining custom char "char"
     repeat 16
       serial.tx(byte[chrDataAddr++])                       ' write character data
  wait100ms

PUB str(strAddr) {{ Print the given string }}
  serial.str(strAddr)

PUB out(char) {{ Print the given character }}
  serial.tx(char)

PUB bin(value, digits) {{ Print a binary number }}
  value <<= 32 - digits
  repeat digits
    out((value <-= 1) & 1 + "0")

PUB hex(value, digits) {{ Print a hexadecimal number }}
  value <<= (8 - digits) << 2
  repeat digits
    out(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))

PUB dec(value) | i {{ Print a decimal number }}
  dec_lj(value, 0)

PUB dec_lj(value, width) | i, c {{ Print a decimal number left justified in a field of width chars }}
  c := 0
  if value < 0
    -value
    out("-")
    c += 1
  elseif value == 0
    out("0")
    c += 1
    repeat while c < width
       c += 1
       out(" ")
    return

  i := 1_000_000_000
  repeat 10
    c += 1
    if value => i
      out(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      out("0")
    i /= 10
  ' now pad out the field with spaces
  repeat while c < width
    c += 1
    out(" ")

PUB dec_rj(value, width) | i, c {{ print a decimal number right justified in a field of width }}
    c := 1
    i := value
    repeat while 9 < i    ' calculate number of digits to print value
       c += 1
       i /= 10
    ' pad out the field with spaces
    repeat while c < width
       c += 1
       out(" ")
    dec_lj(value, 0)

PUB enterBigNum(n) {{ enter bignum mode with the given number of digits (3..4) }}
  if lookdown(n: 3..4)
    serial.str(string("?>"))
    serial.tx("0"+n)
  wait100ms

PUB exitBigNum {{ exit bignum mode }}
  serial.str(string("?<"))

PUB wait100ms
  waitcnt(clkfreq/10+cnt)
